@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
-from graphene import get_version
+from graphene.pyutils.version import get_version
 
+from .directives import all_directives
 from .fields import DjangoObjectField, DjangoFilterListField, DjangoFilterPaginateListField, \
     DjangoListObjectField
+from .middleware import ExtraGraphQLDirectiveMiddleware
 from .mutation import DjangoSerializerMutation
 from .paginations import LimitOffsetGraphqlPagination, PageGraphqlPagination, CursorGraphqlPagination
 from .types import DjangoObjectType, DjangoInputObjectType, DjangoListObjectType, DjangoSerializerType
 
-VERSION = (0, 1, 4, 'final', '')
+VERSION = (0, 4, 3, 'final', '')
 
 __version__ = get_version(VERSION)
 
 __all__ = (
     '__version__',
-    
+
     # FIELDS
     'DjangoObjectField',
     'DjangoFilterListField',
@@ -32,5 +34,9 @@ __all__ = (
     'DjangoObjectType',
     'DjangoListObjectType',
     'DjangoInputObjectType',
-    'DjangoSerializerType'
+    'DjangoSerializerType',
+
+    # DIRECTIVES
+    'all_directives',
+    'ExtraGraphQLDirectiveMiddleware'
 )
