@@ -1,30 +1,74 @@
 # -*- coding: utf-8 -*-
+import inspect
+import re
+import six
 from collections import OrderedDict
 
 from django import VERSION as DJANGO_VERSION
 from django.apps import apps
 from django.contrib.contenttypes.fields import GenericForeignKey, GenericRel
-from django.core.exceptions import ValidationError
-from django.db.models import NOT_PROVIDED, QuerySet, Manager
+from django.core.exceptions import ValidationError, ImproperlyConfigured
+from django.db.models import NOT_PROVIDED, QuerySet, Manager, Model, ManyToOneRel, ManyToManyRel
 from django.db.models.base import ModelBase
 from graphene.utils.str_converters import to_snake_case
-from graphene_django.utils import is_valid_django_model, get_reverse_fields
+from graphene_django.utils import is_valid_django_model
 from graphql import GraphQLList, GraphQLNonNull
-from graphql.language.ast import FragmentSpread
-from rest_framework.compat import _resolve_model
-from six import string_types
+from graphql.language.ast import FragmentSpread, InlineFragment
+
+
+def get_reverse_fields(model):
+    reverse_fields = {
+        f.name: f for f in model._meta.get_fields()
+        if f.auto_created and not f.concrete
+    }
+
+    for name, field in reverse_fields.items():
+        # Django =>1.9 uses 'rel', django <1.9 uses 'related'
+        related = getattr(field, 'rel', None) or \
+            getattr(field, 'related', None)
+        if isinstance(related, ManyToOneRel):
+            yield (name, related)
+        elif isinstance(related, ManyToManyRel) and not related.symmetrical:
+            yield (name, related)
+
+
+def _resolve_model(obj):
+    """
+    Resolve supplied `obj` to a Django model class.
+
+    `obj` must be a Django model class itself, or a string
+    representation of one.  Useful in situations like GH #1225 where
+    Django may not have resolved a string-based reference to a model in
+    another model's foreign key definition.
+
+    String representations should have the format:
+        'appname.ModelName'
+    """
+    if isinstance(obj, six.string_types) and len(obj.split('.')) == 2:
+        app_name, model_name = obj.split('.')
+        resolved_model = apps.get_model(app_name, model_name)
+        if resolved_model is None:
+            msg = "Django did not return a model for {0}.{1}"
+            raise ImproperlyConfigured(msg.format(app_name, model_name))
+        return resolved_model
+    elif inspect.isclass(obj) and issubclass(obj, Model):
+        return obj
+    raise ValueError("{0} is not a Django model".format(obj))
+
+
+def to_kebab_case(name):
+    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1-\2', name.title().replace(' ', ''))
+    return re.sub('([a-z0-9])([A-Z])', r'\1-\2', s1).lower()
 
 
 def get_related_model(field):
     # Backward compatibility patch for Django versions lower than 1.9.x
-    # Function taken from DRF 3.6.x
     if DJANGO_VERSION < (1, 9):
         return _resolve_model(field.rel.to)
     return field.remote_field.model
 
 
 def get_model_fields(model):
-
     # Backward compatibility patch for Django versions lower than 1.11.x
     if DJANGO_VERSION >= (1, 11):
         private_fields = model._meta.private_fields
@@ -36,24 +80,25 @@ def get_model_fields(model):
         list(private_fields) + \
         list(model._meta.fields_map.values())
 
+    # Make sure we don't duplicate local fields with "reverse" version
+    # and get the real reverse django related_name
+    reverse_fields = list(get_reverse_fields(model))
+    exclude_fields = [field[1] for field in reverse_fields]
+
     local_fields = [
         (field.name, field)
         for field
-        in all_fields_list
+        in all_fields_list if field not in exclude_fields
     ]
 
-    # Make sure we don't duplicate local fields with "reverse" version
-    local_field_names = [field[0] for field in local_fields]
-    reverse_fields = get_reverse_fields(model, local_field_names)
-
-    all_fields = local_fields + list(reverse_fields)
+    all_fields = local_fields + reverse_fields
 
     return all_fields
 
 
 def get_obj(app_label, model_name, object_id):
     """
-    Function used by my to get objst
+    Function used to get a object
     :param app_label: A valid Django Model or a string with format: <app_label>.<model_name>
     :param model_name: Key into kwargs that contains de data: new_person
     :param object_id:
@@ -79,10 +124,10 @@ def get_obj(app_label, model_name, object_id):
         raise Exception(e.__str__())
 
 
-def create_obj(model, new_obj_key=None, *args, **kwargs):
+def create_obj(django_model, new_obj_key=None, *args, **kwargs):
     """
     Function used by my on traditional Mutations to create objs
-    :param model: A valid Django Model or a string with format:
+    :param django_model: A valid Django Model or a string with format:
     <app_label>.<model_name>
     :param new_obj_key: Key into kwargs that contains de data: new_person
     :param args:
@@ -91,15 +136,15 @@ def create_obj(model, new_obj_key=None, *args, **kwargs):
     """
 
     try:
-        if isinstance(model, string_types):
-            model = apps.get_model(model)
-        assert is_valid_django_model(model), (
+        if isinstance(django_model, six.string_types):
+            django_model = apps.get_model(django_model)
+        assert is_valid_django_model(django_model), (
             'You need to pass a valid Django Model or a string with format: '
             '<app_label>.<model_name> to "create_obj"'
-            ' function, received "{}".').format(model)
+            ' function, received "{}".').format(django_model)
 
         data = kwargs.get(new_obj_key, None) if new_obj_key else kwargs
-        new_obj = model(**data)
+        new_obj = django_model(**data)
         new_obj.full_clean()
         new_obj.save()
         return new_obj
@@ -214,14 +259,6 @@ def get_Object_or_None(klass, *args, **kwargs):
         # return get_Objects_or_None(klass, *args, **kwargs)
 
 
-def kwargs_formatter(**kwargs):
-    if kwargs.get('deprecation_reason', None):
-        kwargs['deprecation_reason'] = 'DEPRECATED: {}'.format(
-            kwargs['deprecation_reason'])
-
-    return kwargs
-
-
 def get_extra_filters(root, model):
     extra_filters = {}
     for field in model._meta.get_fields():
@@ -267,6 +304,18 @@ def recursive_params(selection_set, fragments, available_related_fields,
             [prefetch_related.append(x)
              for x in b if x not in prefetch_related]
             continue
+        
+        if isinstance(field, InlineFragment):
+            a, b = recursive_params(
+                field.selection_set,
+                fragments,
+                available_related_fields,
+                select_related, prefetch_related
+            )
+            [select_related.append(x) for x in a if x not in select_related]
+            [prefetch_related.append(x)
+             for x in b if x not in prefetch_related]
+            continue
 
         temp = available_related_fields.get(
             field.name.value,
@@ -275,11 +324,10 @@ def recursive_params(selection_set, fragments, available_related_fields,
                 None)
         )
 
-        if temp:
-            if (temp.many_to_many or temp.one_to_many) and \
-               temp.name not in prefetch_related:
+        if temp and temp.name not in [prefetch_related + select_related]:
+            if temp.many_to_many or temp.one_to_many:
                 prefetch_related.append(temp.name)
-            elif temp.name not in select_related:
+            else:
                 select_related.append(temp.name)
         elif getattr(field, 'selection_set', None):
             a, b = recursive_params(
@@ -303,7 +351,7 @@ def queryset_factory(manager, fields_asts=None, fragments=None, **kwargs):
     available_related_fields = get_related_fields(manager.model)
 
     for f in kwargs.keys():
-        temp = available_related_fields.get(f.split('_', 1)[0], None)
+        temp = available_related_fields.get(f.split('__', 1)[0], None)
         if temp:
             if (temp.many_to_many or temp.one_to_many) and \
                temp.name not in prefetch_related:
